#ifndef KQUICKITEMVIEWFXITEM_P_H
#define KQUICKITEMVIEWFXITEM_P_H

#include <QtQuick/private/qtquickglobal_p.h>
#include <QtQuick/private/qquickitem_p.h>
#include <QtQuick/private/qquickitemviewtransition_p.h>
#include <private/qanimationjobutil_p.h>

QT_REQUIRE_CONFIG(quick_itemview);

QT_BEGIN_NAMESPACE

class Q_AUTOTEST_EXPORT KQuickItemViewFxItem
{
public:
    KQuickItemViewFxItem(QQuickItem *item, bool ownItem, QQuickItemChangeListener *changeListener);
    virtual ~KQuickItemViewFxItem();

    qreal itemX() const;
    qreal itemY() const;
    inline qreal itemWidth() const { return item ? item->width() : 0; }
    inline qreal itemHeight() const { return item ? item->height() : 0; }

    void moveTo(const QPointF &pos, bool immediate);
    void setVisible(bool visible);
    void trackGeometry(bool track);

    QRectF geometry() const;
    void setGeometry(const QRectF &geometry);

    QQuickItemViewTransitioner::TransitionType scheduledTransitionType() const;
    bool transitionScheduledOrRunning() const;
    bool transitionRunning() const;
    bool isPendingRemoval() const;

    void transitionNextReposition(QQuickItemViewTransitioner *transitioner, QQuickItemViewTransitioner::TransitionType type, bool asTarget);
    bool prepareTransition(QQuickItemViewTransitioner *transitioner, const QRectF &viewBounds);
    void startTransition(QQuickItemViewTransitioner *transitioner);

    // these are positions and sizes along the current direction of scrolling/flicking
    virtual qreal position() const = 0;
    virtual qreal endPosition() const = 0;
    virtual qreal size() const = 0;
    virtual qreal sectionSize() const = 0;

    virtual bool contains(qreal x, qreal y) const = 0;

    SelfDeletable m_selfDeletable;
    int index = -1;
    QPointer<QQuickItem> item;
    bool ownItem;
    QQuickItemChangeListener *changeListener;
    QQuickItemViewTransitionableItem *transitionableItem;
    bool releaseAfterTransition;
    bool trackGeom;
};

QT_END_NAMESPACE

#endif // KQUICKITEMVIEWFXITEM_P_H
