import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQml 2.12
import QtGraphicalEffects 1.0
import org.ukui.tablet 1.0
import "."

Item {
    id: appIconContainer
    property Item rootItem: undefined
    property int type: LauncherItem.AppIcon
    property string icon: ""
    property var groupModel: undefined
    property int itemId: -1
    property int containerState: LauncherItem.Normal
    property string groupName: ""
    property int deleteIconWidth: 40
    property int dropShadowOffset: 3
    property alias pageContainer: group
    property var mapGroup: iconRect
    property bool allowGroupItemToTaskbar: false
    signal deleteIconSig()
    signal groupItemToDesktop(int itemId, int pageIndex)

    function groupHeight() {
        if (type === LauncherItem.Group) {
            if (modelManager.groupPageCount(itemId) > 1) {
                return (4 * 170 + 13 + 43) * Style.tabletScaleFactor
            }
            return (Math.ceil(modelManager.groupPageItemCount(itemId, 0) / 4) * 170 + 13 + 43) * Style.tabletScaleFactor
        }
        return -1
    }
    function scrollToLastPage() {
        group.pageContainer.currentIndex = group.pageContainer.count - 1
    }

    clip: false

    Rectangle {
        id: underBackground
        width: 112 * Style.tabletScaleFactor
        height: 112 * Style.tabletScaleFactor
        anchors.centerIn: parent
        visible: containerState === LauncherItem.Under
        color: Qt.rgba(0, 0, 0, 0.8)
        radius: 24
    }

    Rectangle {
        id: iconRect
        width: parent.width; height: parent.height
        color: "transparent"
        radius: 24
        z: 1
        property var parentItem: iconRect

        Rectangle {
            id: iconRectBackground
            width: iconRect.width
            height: iconRect.height
            radius: iconRect.radius
            anchors.fill: iconRect
            color: appIconContainer.type === LauncherItem.Group ? "#F7F7F7" : "transparent"
            opacity: appIconContainer.type === LauncherItem.Group ? 0.5 : 1
        }

        DropArea {
            id: dropArea
            anchors.fill: parent
            enabled: containerState === LauncherItem.Popup
            property bool setValue: false
            property int dragItemId: drag.source.itemId
            property int dragPageIndex: drag.source.pageIndex
            LauncherItemView {
                id: group
                clip: true
                groupId: appIconContainer.itemId
                height: {
                    switch (containerState) {
                    case LauncherItem.Popup: return parent.height - 13 * 2 * Style.tabletScaleFactor;
                    default: return 18 * 4 * Style.tabletScaleFactor;
                    }
                }
                width: {
                    switch (containerState) {
                    case LauncherItem.Popup: return parent.width - 13 * 2 * Style.tabletScaleFactor;
                    default: return 18 * 4 * Style.tabletScaleFactor;
                    }
                }
                anchors.centerIn: parent
                visible: type === LauncherItem.Group
                viewMoel: groupModel
                cellWidth: {
                    switch (containerState) {
                    case LauncherItem.Popup: return group.width / 4
                    default: return 18 * Style.tabletScaleFactor;
                    }
                }
                cellHeight: cellWidth
                iconSize: {
                    switch (containerState) {
                    case LauncherItem.Popup: return 96 * Style.tabletScaleFactor;
                    default: return 16 * Style.tabletScaleFactor;
                    }
                }
                allowGroupItemToTaskbar: appIconContainer.allowGroupItemToTaskbar
                Component.onCompleted: {
                    group.launcherItemToDesktop.connect(function() {
                        groupItemToDesktop(dropArea.dragItemId, dropArea.dragPageIndex)
                    })
                }
            }

            PageSwitchIndicator {
                id: pageIndicator
                visible: group.listViewItemCount() > 1 &&
                         type === LauncherItem.Group &&
                         containerState === LauncherItem.Popup
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 16 * Style.tabletScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter
                pageContainer: group.pageContainer
            }
        }
        Image {
            id: appIconImage
            anchors.fill: parent
            anchors.centerIn: parent
            visible: type === LauncherItem.AppIcon
            source: icon
        }
        DropShadow {
            anchors.fill: appIconImage
            horizontalOffset: dropShadowOffset
            radius: dropShadowOffset
            samples: dropShadowOffset * 2 + 1
            color: "#000000"
            opacity: 0.15
            source: appIconImage
        }

        Loader {
            width: Style.appIconGridWidth * widgetColumn
            height: Style.appIconGridHeight * widgetRow
            anchors.centerIn: parent
            source: "file://"+modelData.qmlFile
            visible: type === LauncherItem.Widget
            onLoaded: {
                item.register(item.appname, "tablet-desktop");
            }
        }

        Image {
            id: deleteIcon;
            visible: containerState === LauncherItem.Editable &&
                     type === LauncherItem.AppIcon && modelData.uninstallable
            anchors {
                right: iconRect.right; rightMargin: -(deleteIconWidth/4) * Style.widthScaleFactor;
                top: iconRect.top; topMargin: -(deleteIconWidth/4) * Style.heightScaleFactor;
            }
            source: "qrc:/ui/images/desktop-uninstallbtn.png"
            width: deleteIconWidth * Style.widthScaleFactor
            height: deleteIconWidth * Style.widthScaleFactor
            antialiasing: true
            MouseArea {
                id: deleteMouseArea
                anchors.fill: parent
                onClicked: deleteIconSig()
            }
        }

        GroupTitleEdit {
            id: titleEdit
            title: groupName
            width: parent.width
            height: 92 * Style.tabletScaleFactor
            anchors.bottom: parent.top
            anchors.bottomMargin: 19 * Style.tabletScaleFactor
            visible: false
            onTitleChanged: {
                modelManager.setGroupName(title, itemId)
            }
        }

        state: {
            switch (containerState) {
            case LauncherItem.Normal:
                return "Normal";
            case LauncherItem.Editable:
                return "Editable";
            case LauncherItem.Selectable:
                return "Selectable";
            case LauncherItem.Under:
                return "Under";
            case LauncherItem.Popup:
                return "Popup";
            default: return "Normal"
            }
        }
        states: [
            State {
                name: "Popup"
                ParentChange {
                    target: iconRect; parent: groupPopupLayer;
                    x: (rootItem.width - 706 * Style.tabletScaleFactor) / 2;
                    y: (rootItem.height - groupHeight()) / 2;
                    width: 706 * Style.tabletScaleFactor
                    height: groupHeight()
                }
                PropertyChanges { target: iconRectBackground; color: "#FAFAFA"; opacity: 0.45; }
                PropertyChanges { target: groupPopupBackground; visible: true; }
                PropertyChanges { target: titleEdit; visible: true; }
                StateChangeScript { script: {
                        containerAnimation.stop()
                        iconRect.rotation = 0
                        group.placeholderAdded.connect(function() {
                            iconRect.height = Qt.binding(appIconContainer.groupHeight)
                        })
                    } }
            },
            State {
                name: "Under"
                StateChangeScript { script: { containerAnimation.stop(); iconRect.rotation = 0 } }
                PropertyChanges {
                    target: appIconContainer
                    opacity: 0.5
                }
            },
            State {
                name: "Editable"
                StateChangeScript { script: { containerAnimation.start(); iconRect.scale = 1; } }
            },
            State {
                name: "Normal"
                StateChangeScript {
                    script: {
                        group.update()
                        group.pageContainer.currentIndex = 0
                        containerAnimation.stop()
                        iconRect.rotation = 0
                        iconRect.scale = 1
                        titleEdit.state = "Normal"
                    }
                }
            }
        ]

        transitions: Transition {
            ParentAnimation {
                NumberAnimation { properties: "x,y,width,height"; duration: 200;} }
            onRunningChanged: {
                if (containerState === LauncherItem.Normal) {
                    group.pageContainer.currentIndex = 0
                }
                group.groupAnimationState = running
            }
        }
        Component.onCompleted: {
            group.focusStateChanged.connect(function() {
                titleEdit.state = "Normal"
            })
            group.changeGroupHeight.connect(function() {
                iconRect.height = appIconContainer.groupHeight()
                x = (rootItem.width - 706 * Style.tabletScaleFactor) / 2;
                y = (rootItem.height - groupHeight()) / 2;
            })            
        }
    }

    SequentialAnimation {
        id: containerAnimation;
        loops: AnimatedSprite.Infinite;
        ParallelAnimation {
            RotationAnimation { target: iconRect; from: 0; to: 2; duration: 150; easing.type: Easing.OutCubic; }
        }
        ParallelAnimation {
            RotationAnimation { target: iconRect; from: 2; to: -2; duration: 150; easing.type: Easing.OutCubic; }
        }
    }

    Component.onCompleted: {
    }
}
